@@ -1,18 +1,18 @@
 package Pcore::Core::Event::Listener::file;
 
 use Pcore -class, -ansi, -const;
-use Pcore::Util::Data qw[to_json];
-use Pcore::Util::Scalar qw[is_ref];
+use Pcore::Lib::Data qw[to_json];
+use Pcore::Lib::Scalar qw[is_ref];
 use Fcntl qw[:flock];
 use IO::File;
 use Time::HiRes qw[];
 
 with qw[Pcore::Core::Event::Listener];
 
-has tmpl => '[<: $date.strftime("%Y-%m-%d %H:%M:%S.%4N") :>][<: $channel :>][<: $level :>] <: $title | raw :>' . $LF . '<: $text | raw :>';
+has tmpl => '[<: $date.strftime("%Y-%m-%d %H:%M:%S.%4N") :>][<: $channel :>][<: $level :>] <: $title | raw :>' . "\n" . '<: $text | raw :>';
 
-has _tmpl => ( init_arg => undef );    # InstanceOf ['Pcore::Util::Tmpl']
-has _path => ( init_arg => undef );    # InstanceOf ['Pcore::Util::Path']
+has _tmpl => ( init_arg => undef );    # InstanceOf ['Pcore::Lib::Tmpl']
+has _path => ( init_arg => undef );    # InstanceOf ['Pcore::Lib::Path']
 has _h    => ( init_arg => undef );    # InstanceOf ['IO::File']
 has _init => ( init_arg => undef );
 
@@ -26,7 +26,7 @@ sub forward_event ( $self, $ev ) {
         # init template
         $self->{_tmpl} = P->tmpl;
 
-        $self->{_tmpl}->add_tmpl( message => "$self->{tmpl}$LF" );
+        $self->{_tmpl}->add_tmpl( message => "$self->{tmpl}\n" );
 
         # init path
         if ( $self->{uri}->{path}->{is_abs} ) {
@@ -61,7 +61,7 @@ sub forward_event ( $self, $ev ) {
         if ( defined $ev->{data} ) {
 
             # serialize reference
-            $ev->{text} = is_ref $ev->{data} ? to_json( $ev->{data}, readable => 1 )->$* : $ev->{data};
+            $ev->{text} = is_ref $ev->{data} ? to_json $ev->{data}, readable => 1 : $ev->{data};
 
             # indent
             $ev->{text} =~ s/^/$INDENT/smg;
